###############   PSQF 7375 Advanced Longitudinal Example 3 using R   #############

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import excel XLSX data as table

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("emmeans")) install.packages("emmeans")
library(emmeans) # To get model-implied means

# To estimate MLMs using lmer
# Re-install to fix problem of matrix incompatibility
#oo <- options(repos = "https://cran.r-project.org/")
#utils::install.packages("Matrix")
#utils::install.packages("lme4")
#options(oo)
library(Matrix); library(lme4)

if (!require("lmerTest")) install.packages("lmerTest")
library(lmerTest) # To get Satterthwaite DDF in lmer

if (!require("performance")) install.packages("performance")
library(performance) # To get ICC in lmer

# Clear environment (run as needed)
#rm(list=ls())


################################################################################
#####              BEGIN FUNCTIONS TO AUTOMATE CALCULATIONS                #####
#####       JUST RUN THIS CODE AS IS; SHOULD NOT NEED TO CHANGE IT         #####
################################################################################

# This PseudoR2 function computes pseudo-R2 for each variance from a baseline
#   model to up to two larger models, along with change in pseudo-R2
PseudoR2 = function(data, baseModel, model1, name1, model2=NULL, name2=NULL){
  baseVars = as.data.frame(VarCorr(baseModel))   # Save base model variances
  model1Vars = as.data.frame(VarCorr(model1))    # Save model 1 variances
  test = data.frame(term=baseVars$var1, var2=baseVars$var2,  # Rename and combine
                    base=baseVars$vcov, model1=model1Vars$vcov)
  test1 = test # Save as new name
  test1$pseudoR2.model1 = (test1$base-test1$model1)/test1$base  # Compute pseudo-R2
  test1$term[is.na(test1$term)] = "Residual" # change NA to Residual
  test1 = subset(x=test1, is.na(var2))       # keep if not covariance
  test1 = subset(x=test1, select=-c(var2))   # drop var2
  test1$pseudoR2.model1 = sprintf("%0.4f", test1$pseudoR2.model1)
  if (is.null(model2)) cat(paste("Pseudo-R2 for", name1, "\n"))
  if (is.null(model2)) return(test1)             # Exit if no model2
  model2Vars = as.data.frame(VarCorr(model2))    # Save model 2 variances
  test = data.frame(test, model2=model2Vars$vcov) # Add model 2 variances
  test$pseudoR2.model1 = (test$base-test$model1)/test$base  # Compute pseudo-R2 1
  test$pseudoR2.model2 = (test$base-test$model2)/test$base  # Compute pseudo-R2 2
  test$pseudoR2.change = test$pseudoR2.model2-test$pseudoR2.model1  # Compute change
  test$term[is.na(test$term)] = "Residual" # change NA to Residual
  test = subset(x=test, is.na(var2))       # keep if not covariance
  test = subset(x=test, select=-c(var2))   # drop var2
  test$pseudoR2.model1 = sprintf("%0.4f", test$pseudoR2.model1)
  test$pseudoR2.model2 = sprintf("%0.4f", test$pseudoR2.model2)
  test$pseudoR2.change = sprintf("%0.4f", test$pseudoR2.change)
  cat(paste("Pseudo-R2 and Change in Pseudo-R2 for", name1, "vs", name2, "\n"))
  return(test)
}

# This TotalR2 function computes total-R2 and change in total-R2 
#   using only the fixed effects for up to 2 nested two-level models
TotalR2 = function(data, dvName, model1, name1, model2=NULL, name2=NULL){
  totalR2.1 = as.numeric(cor(data[dvName], predict(model1, re.form=NA))^2)
  cat(paste("Total R2 for", name1, "\n"))
  if (is.null(model2)) return(totalR2.1)
  totalR2.2 = as.numeric(cor(data[dvName], predict(model2, re.form=NA))^2)
  changeR2 = totalR2.2 - totalR2.1
  result = data.frame(totalR2.1, totalR2.2, changeR2)
  cat(paste("Total-R2 and Change in Total-R2 for", name1, "vs", name2, "\n"))
  return(result)
}


################################################################################
#####   BEGIN DATA MANIPULATION OF EXAMPLE 3 ACCELERATED TIME MODELS      #####
#####               CHANGE setwd() TO YOUR FOLDER                         #####
################################################################################

# Set working directory (to import and export files to)
setwd("C:/Dropbox/25_PSQF7375_AdvLong/PSQF7375_AdvLong_Example3_MLM")

# Import Example 3 long-format data from excel -- path = file name
Example3 = read_excel(path="Excel_Chapter10.xlsx", sheet="Chapter10") 
# Convert to data frame to use for analysis
Example3 = as.data.frame(Example3)

# Create time in study (and squared version)
Example3$time=Example3$tvage-Example3$ageT0
Example3$timesq=Example3$time*Example3$time

# Fix 1 case that rounded to 9
Example3$occasion[which(Example3$occasion==9)]=8

# Create rounded age (years since birth) 
Example3$roundage=round(Example3$tvage,digits=0)
# Fix 2 cases above 95
Example3$roundage[which(Example3$roundage>95)]=95

# Center TV age and age at time 0 at 84 years (and squared version)
Example3$tvage84=Example3$tvage-84
Example3$tvage84sq=Example3$tvage84*Example3$tvage84
Example3$ageT084=Example3$ageT0-84
Example3$ageT084sq=Example3$ageT084*Example3$ageT084

############## Labels as comments only #########################
#time      = "time: Years since Time 0"
#timesq    = "timesq: Squared Years since Time 0"
#roundage  = "roundage: Age Rounded to Nearest Year"
#tvage84   = "tvage84: Time-Varying Age (0=84 years)"
#tvage84sq = "tvage84sq: Squared Time-Varying Age (0=84 years)"
#ageT084   = "ageT084: Age at Time 0 (0=84 years)"
#ageT084sq = "ageT084sq: Squared Age at Time 0 (0=84 years)"
#################################################################

# Subset sample to complete cases for all predictors
Example3 = Example3[complete.cases(Example3[ , c("tvage","ageT0","recall")]),]

# Open external file to save results to
txtStart(file="AdvLong_Example3_MLM_R_Output.txt")

###################################################################################
#######      BEGIN DESCRIPTION VIA EMPTY AND SATURATED MEANS MODELS         #######
###################################################################################

print("Descriptive Statistics and Correlations")
 describe(x=Example3[ , c("ageT0","tvage","time","recall")])
corr.test(x=Example3[ , c("ageT0","tvage","time","recall")])

txtComment(" ") # insert blank space in output

print("Empty Means, Random Intercept Model for Age")
Age = lmer(data=Example3, REML=FALSE, formula=tvage~1+(1|PersonID))
summary(Age); icc(Age)
print("Does the random intercept improve model fit?")
ranova(Age, reduce.term=TRUE) # LRT for removing random intercept

txtComment(" ") # insert blank space in output

print("Empty Means, Random Intercept Model for Time")
Time = lmer(data=Example3, REML=FALSE, formula=time~1+(1|PersonID))
summary(Time); icc(Time)
print("Does the random intercept improve model fit?")
ranova(Time, reduce.term=TRUE) # LRT for removing random intercept

txtComment(" ") # insert blank space in output

print("Model 0: Empty Means, Random Intercept Model for Recall Outcome")
Empty = lmer(data=Example3, REML=FALSE, formula=recall~1+(1|PersonID))
llikAIC(Empty); summary(Empty); icc(Empty)
print("Does the random intercept improve model fit?")
ranova(Empty, reduce.term=TRUE) # LRT for removing random intercept

txtComment(" ") # insert blank space in output


###########################################################################################
print("Saturated Means by Rounded Age, Random Intercept Model")
SatAge = lmer(data=Example3, REML=FALSE, formula=recall~0+as.factor(roundage)+(1|PersonID))
summary(SatAge)

# Save and convert list output of means into data frame, compute CIs
SumSatAge = summary(SatAge)
AgeMeans = data.frame(SumSatAge$coefficients)
AgeMeans$LCI = AgeMeans$Estimate-1.96*AgeMeans$Std..Error
AgeMeans$UCI = AgeMeans$Estimate+1.96*AgeMeans$Std..Error
# Extract age from row label from character 20-21
AgeMeans$age = as.numeric(substr(x=rownames(AgeMeans), start=20, stop=21))
AgeMeans # Show result in output

# Save plot: open file, make plot, close file
png(file="R Plots/R Recall by Age.png")  # open file
plot(x=AgeMeans$age, y=AgeMeans$Estimate, type="l",
     ylim=c(min(AgeMeans$LCI), max(AgeMeans$UCI)),
     xlab="Rounded Age in Years", ylab="Predicted Recall")
# Loop over points to add error bars
estimate=1
for (estimate in 1:nrow(AgeMeans)){
  points(x =   AgeMeans$age[estimate], y=AgeMeans$Estimate[estimate])
   lines(x = c(AgeMeans$age[estimate], AgeMeans$age[estimate]), 
         y = c(AgeMeans$LCI[estimate], AgeMeans$UCI[estimate]))
}; dev.off()  # close file

txtComment(" ") # insert blank space in output

###########################################################################################
print("Saturated Means by Rounded Occasion, Random Intercept Model")
SatTim = lmer(data=Example3, REML=FALSE, formula=recall~0+as.factor(occasion)+(1|PersonID))
summary(SatTim)

# Save and convert list output of means into data frame, compute CIs
SumSatTim = summary(SatTim)
TimMeans = data.frame(SumSatTim$coefficients)
TimMeans$LCI = TimMeans$Estimate-1.96*TimMeans$Std..Error
TimMeans$UCI = TimMeans$Estimate+1.96*TimMeans$Std..Error
# Extract occasion from row label from character 20-20
TimMeans$Tim = as.numeric(substr(x=rownames(TimMeans), start=20, stop=20))
TimMeans # Show result in output

# Save plot: open file, make plot, close file
png(file="R Plots/R Recall by Time.png")  # open file
plot(x=TimMeans$Tim, y=TimMeans$Estimate, type="l",
     ylim=c(min(TimMeans$LCI), max(TimMeans$UCI)),
     xlab="Rounded Time in Years", ylab="Predicted Recall")
# Loop over points to add error bars
estimate=1
for (estimate in 1:nrow(TimMeans)){
  points(x =   TimMeans$Tim[estimate], y=TimMeans$Estimate[estimate])
  lines(x = c(TimMeans$Tim[estimate], TimMeans$Tim[estimate]), 
        y = c(TimMeans$LCI[estimate], TimMeans$UCI[estimate]))
}; dev.off()  # close file

txtComment(" ") # insert blank space in output


###########################################################################################
#######                       BEGIN AGE-AS-TIME MODELS                              #######
###########################################################################################

print("Model 1a: Fixed Quadratic Age, Random Intercept Model")
RIAge = lmer(data=Example3, REML=FALSE, formula=recall~1+tvage84+tvage84sq+(1|PersonID))
llikAIC(RIAge); summary(RIAge)

# Total R2 using custom function
TotalR2(data=Example3, dvName="recall", model1=RIAge, name1="TVAge")
# Pseudo-R2 relative to empty model using custom function
PseudoR2(data=Example2, baseModel=Empty, model1=RIAge, name1="TVAge")

txtComment(" ") # insert blank space in output

###########################################################################################
print("Model 2a: Fixed Quadratic Age, Random Intercept Model")
print("Controlling for Birth Cohort as Contextual Effects")
RICohAge = lmer(data=Example3, REML=FALSE, formula=recall~1+tvage84+tvage84sq
                +ageT084+ageT084sq+tvage84:ageT084+(1|PersonID))
llikAIC(RICohAge); summary(RICohAge) 

print("LRT for birth cohort contextual fixed slopes"); anova(RICohAge,RIAge)

print("Linear Combinations to get Total Effects") # g = gamma below
print("Total Linear    Birth Cohort on Intercept"); contest1D(RICohAge, L=c(0,1,0,1,0,0)) # g10+g01
print("Total Quadratic Birth Cohort on Intercept"); contest1D(RICohAge, L=c(0,0,1,0,1,1)) # g20+g02+g11
print("Total Linear Birth Cohort on Linear Slope"); contest1D(RICohAge, L=c(0,0,2,0,0,1)) # 2*g20+g11

# Total R2 and change using custom function
TotalR2(data=Example3, dvName="recall", model1=RIAge, name1="TVage", model2=RICohAge, name2="TVage+Cohort")
# Pseudo-R2 and change relative to empty model using custom function
PseudoR2(data=Example2, baseModel=Empty, model1=RIAge, name1="TVage", model2=RICohAge, name2="TVage+Cohort")

txtComment(" ") # insert blank space in output

###########################################################################################
print("Model 3a: Add Random Linear TVage to Model 2a")
RLCohAge = lmer(data=Example3, REML=FALSE, formula=recall~1+tvage84+tvage84sq
                +ageT084+ageT084sq+tvage84:ageT084+(1+tvage84|PersonID))
llikAIC(RLCohAge); summary(RLCohAge)
print("LRT for random linear TVage slope"); anova(RLCohAge,RICohAge)

txtComment(" ") # insert blank space in output

###########################################################################################
print("Model 4a: Add Random Linear AgeCoh to Model 3a -- won't run at all")
RL2CohAge = lmer(data=Example3, REML=FALSE, formula=recall~1+tvage84+tvage84sq
                 +ageT084+ageT084sq+tvage84:ageT084+(1+tvage84+ageT084|PersonID))
#llikAIC(RL2CohAge); summary(RL2CohAge)
#print("LRT for random baseline age slope"); anova(RL2CohAge,RLCohAge)

txtComment(" ") # insert blank space in output


###########################################################################################
#######                       BEGIN TIME-IN-STUDY MODELS                            #######
###########################################################################################

print("Model 1b: Fixed Quadratic Time, Random Intercept Model")
RITim = lmer(data=Example3, REML=FALSE, formula=recall~1+time+timesq+(1|PersonID))
llikAIC(RITim); summary(RITim)

# Total R2 using custom function
TotalR2(data=Example3, dvName="recall", model1=RITim, name1="Time")
# Pseudo-R2 relative to empty model using custom function
PseudoR2(data=Example2, baseModel=Empty, model1=RITim, name1="Time")

txtComment(" ") # insert blank space in output

###########################################################################################
print("Model 2b: Fixed Quadratic Time, Random Intercept Model")
print("Controlling for Birth Cohort as Total Effects")
RICohTim = lmer(data=Example3, REML=FALSE, formula=recall~1+time+timesq
                +ageT084+ageT084sq+time:ageT084+(1|PersonID))
llikAIC(RICohTim); summary(RICohTim)

print("LRT for birth cohort total fixed slopes"); anova(RICohTim,RITim)

print("Linear Combinations to get Contextual Effects")  # g = gamma
print("Contextual Linear    Birth Cohort on Intercept"); contest1D(RICohTim, L=c(0,-1, 0,1,0, 0)) # g01 - g10
print("Contextual Quadratic Birth Cohort on Intercept"); contest1D(RICohTim, L=c(0, 0, 1,0,1,-1)) # g20 + g01 - g11
print("Contextual Linear Birth Cohort on Linear Slope"); contest1D(RICohTim, L=c(0, 0,-2,0,0, 1)) # g11 - 2*g20

# Total R2 and change using custom function
TotalR2(data=Example3, dvName="recall", model1=RITim, name1="Time", model2=RICohTim, name2="Time+Cohort")
# Pseudo-R2 and change relative to empty model using custom function
PseudoR2(data=Example2, baseModel=Empty, model1=RITim, name1="Time", model2=RICohTim, name2="Time+Cohort")

txtComment(" ") # insert blank space in output

###########################################################################################
print("Model 3b: Add Random Linear Time to Model 2b")
RLCohTim = lmer(data=Example3, REML=FALSE, formula=recall~1+time+timesq
                +ageT084+ageT084sq+time:ageT084+(1+time|PersonID))
llikAIC(RLCohTim); summary(RLCohTim)
print("LRT for random linear time slope"); anova(RLCohTim,RICohTim)

txtComment(" ") # insert blank space in output

###########################################################################################
print("Model 4b: Add Random Linear AgeCoh to Model 3b -- won't run at all")
RL2CohTim = lmer(data=Example3, REML=FALSE, formula=recall~1+time+timesq
                 +ageT084+ageT084sq+time:ageT084+(1+time+ageT084|PersonID))
#llikAIC(RL2CohTim); summary(RL2CohTim)
#print("LRT for random baseline age slope"); anova(RL2CohTim,RLCohTim)

txtComment(" ") # insert blank space in output

###########################################################################################
# Close external file of saved results
txtStop()



