// Prevent "more" messages from appearing
set more off
// Control line length
set linesize 150

*********************************************************************************
******   BEGIN DATA MANIPULATION FOR EXAMPLE 3 ALTERNATIVE TIME MODELS    *******
******                   CHANGE cd to your folder                         *******
*********************************************************************************

// Define working directory for file location to be replaced in code below
// \\Client\ precedes path in Virtual Desktop outside H drive
cd "C:\Dropbox\25_PSQF7375_AdvLong\PSQF7375_AdvLong_Example3_MLM"

// Import Example 3 long data
clear // clear memory in case a dataset is already open
import excel "Excel_Chapter10.xlsx", sheet("Chapter10") firstrow case(preserve) clear 

// Create time in study (and squared version)
gen time = tvage-ageT0
gen timesq = time*time
   
// Fix 1 case rounded to 9
replace occasion=8 if (occasion==9)

// Create rounded age (years since birth)
gen roundage = round(tvage,1)
// Fix 2 cases above 95
replace roundage=95 if (roundage==97)
replace roundage=95 if (roundage==100)

// Center TV age and age at time 0 at 84 (and squared versions)
gen tvage84 = tvage-84
gen tvage84sq = tvage84*tvage84
gen ageT084 = ageT0-84
gen ageT084sq = ageT084*ageT084

// Label variables
label variable time       "time: Years since Time 0"
label variable timesq     "timesq: Squared Years since Time 0"
label variable roundage   "roundage: Age Rounded to Nearest Year"
label variable tvage84    "tvage84: Time-Varying Age (0=84 years)"
label variable tvage84sq  "tvage84sq: Squared Time-Varying Age (0=84 years)"
label variable ageT084    "ageT084: Age at Time 0 (0=84 years)"
label variable ageT084sq  "ageT084: Squared Age at Time 0 (0=84 years)"

// Subset sample to complete cases for all predictors
egen nummiss = rowmiss(tvage ageT0 recall)
drop if nummiss>0

// Open external file to save results to
log using AdvLong_Example3_MLM_STATA_Output.log, replace

    
********************************************************************************
******    BEGIN DESCRIPTION VIA EMPTY AND SATURATED MEANS MODELS         *******
********************************************************************************

display "Descriptive Statistics and Correlations"
summarize ageT0 tvage time recall
pwcorr ageT0 tvage time recall, sig

display "Empty Means, Random Intercept Model for Age"
mixed tvage , || PersonID: , mle nolog 
estat icc  // Intraclass correlation
          
display "Empty Means, Random Intercept Model for Time"
mixed time , || PersonID: , mle nolog
estat icc  // Intraclass correlation

display "Model 0: Empty Means, Random Intercept Model for Recall Outcome"
mixed recall , || PersonID: , mle nolog
matrix Empty = r(table)     // Save results for computations below
display "-2LL = " e(ll)*-2  // Print -2LL for model  
estat icc  // Intraclass correlation
          
display "Saturated Means by Rounded Age, Random Intercept Model"
mixed recall i.roundage, || PersonID: , mle nolog baselevel
margins  i.roundage  // get saturated means per age and plot them
marginsplot, xdimension(roundage) name(by_age, replace) 
graph export "STATA plots\STATA Recall by Age.png", replace

display "Saturated Means by Rounded Time, Random Intercept Model"
mixed recall i.occasion, || PersonID: , mle nolog baselevel
margins  i.occasion  // get saturated means per occasion and plot them
marginsplot, xdimension(occasion) name(by_time, replace) 
graph export "STATA plots\STATA Recall by Time.png", replace

          
********************************************************************************
******                BEGIN AGE-AS-TIME MODELS                           *******
********************************************************************************

display "Model 1a: Fixed Quadratic Age, Random Intercept Model"
mixed recall c.tvage84 c.tvage84sq, || PersonID: , mle nolog 
matrix RIAge = r(table)     // Save results for computations below
display "-2LL = " e(ll)*-2  // Print -2LL for model  
estimates store FitRIAge    // Save for LRT

// Build and save total-R2
predict PredAge, xb             // Save fixed-effect predicted outcome
quietly corr recall PredAge     // Get total r to make R2
global R2Age = r(rho)^2         // Save total-R2 for comparison
display "Total-R2 = " $R2Age    // Print total-R2 relative to empty model

// Build pseudo-R2
matrix list RIAge  // Show saved results (variances are saved as log of SD)
display "Pseudo-R2 for Intercept = " 1-(exp(RIAge[1,4])^2/exp(Empty[1,2])^2)
display "Pseudo-R2 for Residual = "  1-(exp(RIAge[1,5])^2/exp(Empty[1,3])^2)


////////////////////////////////////////////////////////////////////////////////
display "Model 2a: Fixed Quadratic Age, Random Intercept Model"
display "Controlling for Birth Cohort as Contextual Effects"
mixed recall c.tvage84 c.tvage84sq c.ageT084 c.ageT084sq c.tvage84#c.ageT084, ///
             || PersonID: , mle nolog 
matrix RICohAge = r(table)  // Save results for computations below
display "-2LL = " e(ll)*-2  // Print -2LL for model  

estimates store FitRICohAge     // Save for LRT
lrtest FitRICohAge FitRIAge     // LRT for birth cohort contextual fixed slopes

// Linear Combinations to get Total Effects (g = gamma below)
// Total Linear Birth Cohort on Intercept = g10+g01
   lincom c.ageT084*1 + c.tvage84*1
// Total Quadratic Birth Cohort on Intercept = 2g20+g11
   lincom c.ageT084sq*1 + c.tvage84#c.ageT084*1 + c.tvage84sq*1
// Total Linear Birth Cohort on Linear Slope = 2*g20+g11
   lincom c.tvage84#c.ageT084*1 + c.tvage84sq*2
   
// Build and save total-R2
predict PredCohAge, xb          // Save fixed-effect predicted outcome
quietly corr recall PredCohAge  // Get total r to make R2
global R2CohAge = r(rho)^2      // Save total-R2 for comparison
display "Total-R2 = " $R2CohAge // Print total-R2 relative to empty model
display "Change in Total-R2 = " $R2CohAge - $R2Age

// Build pseudo-R2
matrix list RICohAge  // Show saved results (variances are saved as log of SD)
display "Pseudo-R2 for Intercept = " 1-(exp(RICohAge[1,7])^2/exp(Empty[1,2])^2)
display "Pseudo-R2 for Residual = "  1-(exp(RICohAge[1,8])^2/exp(Empty[1,3])^2)

display "Change in Pseudo-R2 for Intercept = " ///
(1-(exp(RICohAge[1,7])^2/exp(Empty[1,2])^2)) - (1-(exp(RIAge[1,4])^2/exp(Empty[1,2])^2))

display "Change in Pseudo-R2 for Residual = " ///
(1-(exp(RICohAge[1,8])^2/exp(Empty[1,3])^2)) - (1-(exp(RIAge[1,5])^2/exp(Empty[1,3])^2))          
		  
////////////////////////////////////////////////////////////////////////////////
display "Model 3a: Add Random Linear TVage to Model 2a"
mixed recall c.tvage84 c.tvage84sq c.ageT084 c.ageT084sq c.tvage84#c.ageT084, ///
             || PersonID: tvage84, mle nolog covariance(unstructured)
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat recovariance, relevel(PersonID) correlation  // GCORR matrix
estimates store FitRLCohAge     // Save for LRT
lrtest FitRLCohAge FitRICohAge  // LRT for random linear TVage slope       


////////////////////////////////////////////////////////////////////////////////
display "Model 4a: Add Random Linear AgeCoh to Model 3a"
display "Converged onto NPD solution even after extra iterations"
mixed recall c.tvage84 c.tvage84sq c.ageT084 c.ageT084sq c.tvage84#c.ageT084, ///
             || PersonID: tvage84 ageT084, mle nolog covariance(unstructured) ///
			 emiterate(100) 
display "-2LL = " e(ll)*-2         // Print -2LL for model  
estat recovariance, relevel(PersonID) correlation  // GCORR matrix

// Gives error message for LRT
//estimates store FitRL2CohAge     // Save for LRT
//lrtest FitRL2CohAge FitRLCohAge  // LRT for random linear baseline age slope        
          

********************************************************************************
******                  BEGIN TIME-IN-STUDY MODELS                       *******
********************************************************************************

display "Model 1b: Fixed Quadratic Time, Random Intercept Model"
mixed recall c.time c.timesq, || PersonID: , mle nolog 
matrix RITim = r(table)     // Save results for computations below
display "-2LL = " e(ll)*-2  // Print -2LL for model  
estimates store FitRITim    // Save for LRT

// Build and save total-R2
predict PredTim, xb             // Save fixed-effect predicted outcome
quietly corr recall PredTim     // Get total r to make R2
global R2Tim = r(rho)^2         // Save total-R2 for comparison
display "Total-R2 = " $R2Tim    // Print total-R2 relative to empty model

// Build pseudo-R2
matrix list RITim  // Show saved results (variances are saved as log of SD)
display "Pseudo-R2 for Intercept = " 1-(exp(RITim[1,4])^2/exp(Empty[1,2])^2)
display "Pseudo-R2 for Residual = "  1-(exp(RITim[1,5])^2/exp(Empty[1,3])^2)
     

//////////////////////////////////////////////////////////////////////////////
display "Model 2b: Fixed Quadratic Time, Random Intercept Model"
display "Controlling for Birth Cohort as Total Effects"
mixed recall time c.timesq c.ageT084 c.ageT084sq c.time#c.ageT084, ///
             || PersonID: , mle nolog
matrix RICohTim = r(table)  // Save results for computations below
display "-2LL = " e(ll)*-2  // Print -2LL for model  

estimates store FitRICohTim     // Save for LRT
lrtest FitRICohTim FitRITim     // LRT for birth cohort total fixed slopes

// Linear Combinations to get Contextual Effects (g = gamma)
// Contextual Linear Birth Cohort on Intercept = g01 - g10
   lincom c.ageT084*1 + time*-1
// Contextual Quadratic Birth Cohort on Intercept = g20 + g01 - g11
   lincom c.ageT084sq*1 + c.time#c.ageT084*-1 + c.timesq*1
// Contextual Linear Birth Cohort on Linear Slope = g11 - 2*g20
   lincom c.time#c.ageT084*1 + c.timesq*-2
   
// Build and save total-R2
predict PredCohTim, xb          // Save fixed-effect predicted outcome
quietly corr recall PredCohTim  // Get total r to make R2
global R2CohTim = r(rho)^2      // Save total-R2 for comparison
display "Total-R2 = " $R2CohTim // Print total-R2 relative to empty model
display "Change in Total-R2 = " $R2CohTim - $R2Tim

// Build pseudo-R2
matrix list RICohAge  // Show saved results (variances are saved as log of SD)
display "Pseudo-R2 for Intercept = " 1-(exp(RICohAge[1,7])^2/exp(Empty[1,2])^2)
display "Pseudo-R2 for Residual = "  1-(exp(RICohAge[1,8])^2/exp(Empty[1,3])^2)

display "Change in Pseudo-R2 for Intercept = " ///
(1-(exp(RICohTim[1,7])^2/exp(Empty[1,2])^2)) - (1-(exp(RITim[1,4])^2/exp(Empty[1,2])^2))

display "Change in Pseudo-R2 for Residual = " ///
(1-(exp(RICohTim[1,8])^2/exp(Empty[1,3])^2)) - (1-(exp(RITim[1,5])^2/exp(Empty[1,3])^2))          
		 
          
/////////////////////////////////////////////////////////////////////////////
display "Model 3b: Add Random Linear Time to Model 2b"
mixed recall c.time c.timesq c.ageT084 c.ageT084sq c.tvage84#c.ageT084, ///
             || PersonID: time, mle nolog covariance(unstructured)
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat recovariance, relevel(PersonID) correlation  // GCORR matrix

estimates store FitRLCohTim     // Save for LRT
lrtest FitRLCohTim FitRICohTim  // LRT for random linear time slope
              
          
////////////////////////////////////////////////////////////////////////////
display "Model 4b: Add Random Linear AgeCoh to Model 3b"
display "Converged onto NPD solution even after extra iterations"
mixed recall c.time c.timesq c.ageT084 c.ageT084sq c.tvage84#c.ageT084,    ///
             || PersonID: time ageT084, mle nolog covariance(unstructured) ///
			 emiterate(100)
display "-2LL = " e(ll)*-2       // Print -2LL for model  
estat recovariance, relevel(PersonID) correlation  // GCORR matrix

estimates store FitRL2CohTim     // Save for LRT
lrtest FitRL2CohTim FitRLCohTim  // LRT for random linear baseline age slope                              
          
		  
///////////////////////////////////////////////////////////////////////////          
// Close external file of saved results
log close

// Remove all macros (so they don't get used in other STATA programs)
macro drop _all 
